package cn.com.ty.lift.business.emergency.controller;


import cn.com.ty.lift.business.common.CountPage;
import cn.com.ty.lift.business.common.Judge;
import cn.com.ty.lift.business.common.Values;
import cn.com.ty.lift.business.emergency.dto.LiftFaultCategory;
import cn.com.ty.lift.business.emergency.dto.LiftFaultType;
import cn.com.ty.lift.business.emergency.dto.RepairRequest;
import cn.com.ty.lift.business.emergency.dto.RepairResponse;
import cn.com.ty.lift.business.emergency.entity.EmergencyRepair;
import cn.com.ty.lift.business.emergency.entity.ErCostItem;
import cn.com.ty.lift.business.emergency.entity.LiftFault;
import cn.com.ty.lift.business.emergency.service.EmergencyRepairService;
import cn.com.ty.lift.business.emergency.service.ErCostItemService;
import cn.com.ty.lift.business.emergency.service.LiftFaultService;
import cn.com.ty.lift.business.evaluation.dao.entity.Evaluation;
import cn.com.ty.lift.business.evaluation.dao.entity.model.EvaluationRequest;
import cn.com.ty.lift.business.evaluation.service.EvaluationService;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 临修记录 前端控制器
 * </p>
 *
 * @author wcz
 * @since 2019-12-04
 */
@Slf4j
@RestController
@AllArgsConstructor
@RequestMapping("/emergency/emergency-repair")
public class EmergencyRepairController {

    private EmergencyRepairService emergencyRepairService;

    private ErCostItemService erCostItemService;

    private LiftFaultService liftFaultService;

    private EvaluationService evaluationService;

    /**
     * @description 根据id查找
     * @date 2019/12/6 10:14
     * @param
     * @return
     */
    @PostMapping("findOne")
    public RestResponse findOne(@RequestBody RepairRequest request){
        Long id = request.getId();
        Judge.id(id);
        RepairResponse result = emergencyRepairService.selectByIdWithInfo(request);
        Judge.notNull(result);
        //查询相关的急修费用
        List<ErCostItem> erCostItems = erCostItemService.listByErRecordId(id);
        result.setErCostItems(erCostItems);
        //故障信息
        String faultPart = result.getFaultPart();
        String faultReason = result.getFaultReason();
        String faultNature = result.getFaultNature();
        String faultHandle = result.getFaultHandle();
        String faultDuty = result.getFaultDuty();
        if(StrUtil.isNotEmpty(faultPart)){
            List<Long> partIds = Arrays.asList(faultPart.split(",")).stream().map(s -> Long.parseLong(s.trim())).collect(Collectors.toList());
            result.setFaultParts(liftFaultService.listByIds(partIds));
        }
        if(StrUtil.isNotEmpty(faultReason)){
            List<Long> reasonIds = Arrays.asList(faultPart.split(",")).stream().map(s -> Long.parseLong(s.trim())).collect(Collectors.toList());
            result.setFaultReasons(liftFaultService.listByIds(reasonIds));
        }
        if(StrUtil.isNotEmpty(faultNature)){
            List<Long> natureIds = Arrays.asList(faultPart.split(",")).stream().map(s -> Long.parseLong(s.trim())).collect(Collectors.toList());
            result.setFaultNatures(liftFaultService.listByIds(natureIds));
        }
        if(StrUtil.isNotEmpty(faultHandle)){
            List<Long> handleIds = Arrays.asList(faultPart.split(",")).stream().map(s -> Long.parseLong(s.trim())).collect(Collectors.toList());
            result.setFaultHandles(liftFaultService.listByIds(handleIds));
        }
        if(StrUtil.isNotEmpty(faultDuty)){
            List<Long> dutyIds = Arrays.asList(faultPart.split(",")).stream().map(s -> Long.parseLong(s.trim())).collect(Collectors.toList());
            result.setFaultDuties(liftFaultService.listByIds(dutyIds));
        }
        result.setEvaluation(evaluationService.findByRecord(id));
        return RestResponse.success(result);
    }

    /**
     * @description 新增急修
     * @param
     * @return
     * @date 2019/12/9 9:43
     */
    @PostMapping("add")
    public RestResponse add(@RequestBody EmergencyRepair entity){
        int repairing = emergencyRepairService.countRepairing(entity.getMtCompanyId(),entity.getLiftId());
        Judge.notTrue(repairing > 0,"该电梯存在未完成的急修，无法创建");
        //派单时间 急修单创建时间
        entity.setAssignTime(LocalDateTime.now());
        entity.setStatus(Values.ER_STATUS_TO_REPAIRED);
        boolean result = emergencyRepairService.save(entity);
        return Judge.result(result);
    }

    /**
     * @description 分页查询急修中的 status = 2
     * @date 2019/12/9 9:39
     * @param 
     * @return 
     */
    @PostMapping("list")
    public RestResponse list(@RequestBody RepairRequest request){
        IPage<RepairResponse> pages = emergencyRepairService.pageByRepairing(request);
        Judge.notNull(pages);
        long count = pages.getTotal();
        CountPage<RepairResponse> countPage = new CountPage<>(count,pages);
        return RestResponse.success(countPage);
    }

    /**
     * @description 根据条件分页查询
     * @date 2019/12/9 9:39
     * @param 
     * @return 
     */
    @PostMapping("page")
    public RestResponse page(@RequestBody RepairRequest request){
        IPage<RepairResponse> pages = emergencyRepairService.pageByCondition(request);
        Judge.notNull(pages);
        long count = emergencyRepairService.countByCondition(request);
        CountPage<RepairResponse> countPage = new CountPage<>(count,pages);
        return RestResponse.success(countPage);
    }

    /**
     * @description 根据条件分页查询
     * @date 2019/12/9 9:39
     * @param
     * @return
     */
    @PostMapping("pageWorker")
    public RestResponse pageWorker(@RequestBody RepairRequest request){
        Long workerId = request.getWorkerId();
        Judge.id(workerId);
        IPage<RepairResponse> pages = emergencyRepairService.pageByCondition(request);
        return Judge.result(pages);
    }

    /**
     * @description 急修转派
     * @date 2019/12/11 9:25
     * @param
     * @return
     */
    @PostMapping("transfer")
    public RestResponse transfer(@RequestBody RepairRequest request){
        Long id = request.getId();
        Long workerId = request.getWorkerId();
        Judge.ids(id,workerId);
        EmergencyRepair entity = emergencyRepairService.getById(id);
        Judge.notNull(entity);
        //如果已经接单，在急修中了不能被转派
        Judge.notTrue(Values.erStatusRepairing(entity.getStatus()),"急修已经接单，不能转派");
        //本急修单的负责维保工
        entity.setWorkerId1(workerId);
        boolean result = emergencyRepairService.saveOrUpdate(entity);
        return Judge.result(result);
    }

    /**
     * @description 关闭急修
     * @date 2019/12/11 9:25
     * @param
     * @return
     */
    @PostMapping("close")
    public RestResponse close(@RequestBody RepairRequest request){
        Long id = request.getId();
        //fixme:assert判断条件
        Judge.id(id);
        String reason = request.getCloseReason();
        Judge.notNull(reason,"缺少关闭原因");
        EmergencyRepair entity = emergencyRepairService.getById(id);
        Judge.notNull(entity);
        entity.setCloseReason(reason);
        //新状态（-1 暂停中，0 待修理；1 修理中；2 已完成 3 已关闭）
        entity.setStatus(Values.ER_STATUS_CLOSE);
        boolean result = emergencyRepairService.saveOrUpdate(entity);
        return Judge.result(result);
    }

    /**
     * app-其他角色获取自己关联的项目的急修任务
     * @param request
     * @return
     */
    @PostMapping("pageByUser")
    public RestResponse pageByUser(@RequestBody RepairRequest request){
        Long userId = request.getUserId();
        Judge.id(userId);
        IPage<RepairResponse> pages = emergencyRepairService.pageByUser(request);
        return Judge.result(pages);
    }

    /**
     * app-点击接单，保存接单时间到急修表的接单时间接口
     * 接单的时候就是签到
     * @param request
     * @return
     */
    @PostMapping("taking")
    public RestResponse taking(@RequestBody RepairRequest request){
        Long id = request.getId();
        Judge.id(id);
        LocalDateTime takingTime = request.getTakingTime();
        Judge.notNull(takingTime,"缺少接单时间");
        EmergencyRepair entity = emergencyRepairService.getById(id);
        Judge.notNull(entity);
        Judge.isTrue(Values.erStatusToRepaired(entity.getStatus()),"待急修的才能接单");
        entity.setTakingTime(takingTime);
        //状态改成修理中 todo:修改电梯的业务状态为急修中,如果存在维保中，要修改
        entity.setStatus(Values.ER_STATUS_REPAIRING);
        boolean result = emergencyRepairService.taking(entity);
        return Judge.result(result);
    }

    /**
     * app-点击到达，保存到达时间到急修表的到达时间中
     * @param request
     * @return
     */
    @PostMapping("arrive")
    public RestResponse arrive(@RequestBody RepairRequest request){
        Long id = request.getId();
        Judge.id(id);
        LocalDateTime arriveTime = request.getArriveTime();
        Judge.notNull(arriveTime,"缺少达到时间");
        String position = request.getPosition();
        Judge.notNull(position,"缺少达到地点");
        EmergencyRepair entity = emergencyRepairService.getById(id);
        Judge.notNull(entity);
        Judge.isTrue(Values.erStatusRepairing(entity.getStatus()), "急修中的才能到达");
        entity.setArriveTime(arriveTime);
        boolean result = emergencyRepairService.updateById(entity);
        return Judge.result(result);
    }

    /**
     * app-停梯时间+安全确认，保存停梯时间+安全确认项到急修表中
     * 修改电梯的业务状态为急修中
     * @param request
     * @return
     */
    @PostMapping("stop")
    public RestResponse stopLift(@RequestBody RepairRequest request){
        Long id = request.getId();
        Judge.id(id);
        LocalDateTime stopDate = request.getStopDate();
        Judge.notNull(stopDate,"请指定停梯时间");
        String safetyConfirm = request.getSafetyConfirm();
        Judge.notNull(safetyConfirm,"请完成安全确认");
        EmergencyRepair entity = emergencyRepairService.getById(id);
        Judge.notNull(entity);
        Judge.isTrue(Values.erStatusRepairing(entity.getStatus()),"急修中的才能停梯");
        entity.setStopDate(stopDate);
        entity.setSafetyConfirm(safetyConfirm);
        boolean result = emergencyRepairService.updateById(entity);
        return Judge.result(result);
    }

    /**
     * @description 新增急修收费单项
     * @date 2019/12/9 17:23
     * @param
     * @return
     */
    @PostMapping("cost/add")
    public RestResponse costAdd(@RequestBody RepairRequest request){
        Long id = request.getId();
        Judge.id(id);
        List<ErCostItem> erCostItems = request.getErCostItems();
        Judge.notNull(erCostItems,"缺少收费项");
        BigDecimal costTotal = new BigDecimal(0);
        erCostItems.forEach(item -> {
            costTotal.add(item.getCostMoney());
        });
        boolean result = erCostItemService.saveBatch(erCostItems);
        return Judge.result(result);
    }

    /**
     * 删除急修下的收费项
     * @param request
     * @return
     */
    @PostMapping("cost/delete")
    public RestResponse costDelete(@RequestBody RepairRequest request){
        Long erCostItemId = request.getErCostItemId();
        Judge.id(erCostItemId);
        boolean result = erCostItemService.removeById(erCostItemId);
        return Judge.result(result);
    }

    /**
     * 修改急修单下的急修收费项
     * @param request
     * @return
     */
    @PostMapping("cost/modify")
    public RestResponse costModify(@RequestBody RepairRequest request){
        List<ErCostItem> erCostItems = request.getErCostItems();
        Judge.notNull(erCostItems);
        boolean result = erCostItemService.updateBatchById(erCostItems);
        return Judge.result(result);
    }

    /**
     * 根据电梯类型查询故障信息（集合结构）
     * @param request
     * @return
     */
    @PostMapping("fault/list")
    public RestResponse faultList(@RequestBody RepairRequest request){
        Integer liftCategory = request.getLiftCategory();
        if(null == liftCategory || liftCategory < 0){
            return RestResponse.success(liftFaultService.list());
        }
        List<LiftFault> lists = liftFaultService.listByLiftCategory(liftCategory);
        return Judge.result(lists);
    }

    /**
     * 根据电梯类型获取急修故障信息列表(树结构)
     * @param request
     * @return
     */
    @PostMapping("fault/tree")
    public RestResponse faultTree(@RequestBody RepairRequest request) {
        Integer liftCategory = request.getLiftCategory();
        if(null == liftCategory || liftCategory < 0){
            List<LiftFault> liftFaults = liftFaultService.list();
            return assemble(liftFaults);
        }else{
            List<LiftFault> liftFaults = liftFaultService.listByLiftCategory(liftCategory);
            return assemble(liftFaults);
        }
    }

    /**
     * 根据故障列表组装tree
     * @param liftFaults
     * @return
     */
    private RestResponse assemble(List<LiftFault> liftFaults){
        Judge.notNull(liftFaults,"没有故障信息");
        List<String> categories = liftFaults.stream().map(LiftFault::getFaultCategory).distinct().collect(Collectors.toList());
        Judge.notNull(categories,"没有故障类型");
        List<LiftFaultCategory> liftFaultCategories = new ArrayList<>();
        categories.forEach(category -> {
            log.info("category: {}", category);
            List<String> types = liftFaults.stream().filter(liftFault -> (
                    StrUtil.equals(category,liftFault.getFaultCategory())
            )).map(LiftFault::getFaultType).distinct().collect(Collectors.toList());
            log.info("types: {}", types);
            List<LiftFaultType> liftFaultTypes = new ArrayList<>();
            if (ObjectUtil.isNotEmpty(types)) {
                types.forEach(type -> {
                    log.info("type: {}",type);
                    List<LiftFault> faults = liftFaults.stream().filter(liftFault -> (
                            StrUtil.equals(category, liftFault.getFaultCategory()) &&
                                    StrUtil.equals(type, liftFault.getFaultType())
                    )).distinct().collect(Collectors.toList());
                    liftFaultTypes.add(new LiftFaultType(type,faults));
                });
            }
            liftFaultCategories.add(new LiftFaultCategory(category,liftFaultTypes));
        });
        return RestResponse.success(liftFaultCategories);
    }

    /**
     * 急修评价
     * @param request
     * @return
     */
    @PostMapping("evaluate")
    public RestResponse evaluate(@RequestBody EvaluationRequest request){
        Long recordId = request.getRecordId();
        Judge.id(recordId);
        String customerSign = request.getCustomerSign();
        Judge.notNull(customerSign,"请签名");
        EmergencyRepair repair = emergencyRepairService.getById(recordId);
        Judge.notNull(repair);
        Judge.isTrue(Values.erStatusFinish(repair.getStatus()),"该急修未完成不能评价");
        int count = evaluationService.countByRecord(recordId);
        Judge.notTrue(count > 0,"该急修已经存在评价");
        Evaluation evaluation = new Evaluation();
        //复制属性到评价bean
        BeanUtil.copyProperties(request,evaluation);
        evaluation.setPostDate(LocalDateTime.now());
        boolean result = evaluationService.save(evaluation);
        return Judge.result(result);
    }

    /**
     * 生成急修单，添加主要，次要急修人签名
     * @param request
     * @return
     */
    @PostMapping("repairOrder")
    public RestResponse repairOrder(@RequestBody RepairRequest request){
        Long id = request.getId();
        Judge.id(id);
        LocalDateTime recoveryDate = request.getRecoveryDate();
        Judge.notNull(recoveryDate,"请填写恢复时间");
        String mainSign = request.getMainSign();
        String secondSign = request.getSecondSign();
        Judge.notTrue(StrUtil.hasEmpty(mainSign,secondSign),"请签名");
        EmergencyRepair entity = emergencyRepairService.getById(id);
        Judge.notNull(entity);
        entity.setFaultPart(request.getFaultPart());
        entity.setFaultReason(request.getFaultReason());
        entity.setFaultHandle(request.getFaultHandle());
        entity.setFaultNature(request.getFaultNature());
        entity.setFaultDuty(request.getFaultDuty());
        entity.setRecoveryDate(recoveryDate);

        //修改状态已完成
        entity.setStatus(Values.ER_STATUS_FINISH);
        boolean result = emergencyRepairService.updateById(entity);
        return Judge.result(result);
    }
}
